@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 2.1.  */
+/* A Bison parser, made by GNU Bison 1.875c.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
 /* As a special exception, when this file is copied by Bison into a
    Bison output file, you may use that output file without restriction.
@@ -36,9 +36,6 @@
 /* Identify Bison output.  */
 #define YYBISON 1
 
-/* Bison version.  */
-#define YYBISON_VERSION "2.1"
-
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
 
@@ -93,7 +90,6 @@
      QUOTED_STRING = 292
    };
 #endif
-/* Tokens.  */
 #define CHECK 258
 #define CODESTART 259
 #define COPYRIGHT 260
@@ -259,19 +255,14 @@ static char *xstrdup (const char *);
 # define YYERROR_VERBOSE 0
 #endif
 
-/* Enabling the token table.  */
-#ifndef YYTOKEN_TABLE
-# define YYTOKEN_TABLE 0
-#endif
-
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
 #line 113 "nlmheader.y"
 typedef union YYSTYPE {
   char *string;
   struct string_list *list;
 } YYSTYPE;
-/* Line 196 of yacc.c.  */
-#line 275 "nlmheader.c"
+/* Line 191 of yacc.c.  */
+#line 266 "nlmheader.c"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -282,49 +273,30 @@ typedef union YYSTYPE {
 /* Copy the second part of user declarations.  */
 
 
-/* Line 219 of yacc.c.  */
-#line 287 "nlmheader.c"
-
-#if ! defined (YYSIZE_T) && defined (__SIZE_TYPE__)
-# define YYSIZE_T __SIZE_TYPE__
-#endif
-#if ! defined (YYSIZE_T) && defined (size_t)
-# define YYSIZE_T size_t
-#endif
-#if ! defined (YYSIZE_T) && (defined (__STDC__) || defined (__cplusplus))
-# include <stddef.h> /* INFRINGES ON USER NAME SPACE */
-# define YYSIZE_T size_t
-#endif
-#if ! defined (YYSIZE_T)
-# define YYSIZE_T unsigned int
-#endif
-
-#ifndef YY_
-# if YYENABLE_NLS
-#  if ENABLE_NLS
-#   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
-#   define YY_(msgid) dgettext ("bison-runtime", msgid)
-#  endif
-# endif
-# ifndef YY_
-#  define YY_(msgid) msgid
-# endif
-#endif
+/* Line 214 of yacc.c.  */
+#line 278 "nlmheader.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
+# ifndef YYFREE
+#  define YYFREE free
+# endif
+# ifndef YYMALLOC
+#  define YYMALLOC malloc
+# endif
+
 /* The parser invokes alloca or malloc; define the necessary symbols.  */
 
 # ifdef YYSTACK_USE_ALLOCA
 #  if YYSTACK_USE_ALLOCA
+#   define YYSTACK_ALLOC alloca
+#  endif
+# else
+#  if defined (alloca) || defined (_ALLOCA_H)
+#   define YYSTACK_ALLOC alloca
+#  else
 #   ifdef __GNUC__
 #    define YYSTACK_ALLOC __builtin_alloca
-#   else
-#    define YYSTACK_ALLOC alloca
-#    if defined (__STDC__) || defined (__cplusplus)
-#     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
-#     define YYINCLUDED_STDLIB_H
-#    endif
 #   endif
 #  endif
 # endif
@@ -332,39 +304,13 @@ typedef union YYSTYPE {
 # ifdef YYSTACK_ALLOC
    /* Pacify GCC's `empty if-body' warning. */
 #  define YYSTACK_FREE(Ptr) do { /* empty */; } while (0)
-#  ifndef YYSTACK_ALLOC_MAXIMUM
-    /* The OS might guarantee only one guard page at the bottom of the stack,
-       and a page size can be as small as 4096 bytes.  So we cannot safely
-       invoke alloca (N) if N exceeds 4096.  Use a slightly smaller number
-       to allow for a few compiler-allocated temporary stack slots.  */
-#   define YYSTACK_ALLOC_MAXIMUM 4032 /* reasonable circa 2005 */
-#  endif
 # else
+#  if defined (__STDC__) || defined (__cplusplus)
+#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
+#   define YYSIZE_T size_t
+#  endif
 #  define YYSTACK_ALLOC YYMALLOC
 #  define YYSTACK_FREE YYFREE
-#  ifndef YYSTACK_ALLOC_MAXIMUM
-#   define YYSTACK_ALLOC_MAXIMUM ((YYSIZE_T) -1)
-#  endif
-#  ifdef __cplusplus
-extern "C" {
-#  endif
-#  ifndef YYMALLOC
-#   define YYMALLOC malloc
-#   if (! defined (malloc) && ! defined (YYINCLUDED_STDLIB_H) \
-	&& (defined (__STDC__) || defined (__cplusplus)))
-void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
-#   endif
-#  endif
-#  ifndef YYFREE
-#   define YYFREE free
-#   if (! defined (free) && ! defined (YYINCLUDED_STDLIB_H) \
-	&& (defined (__STDC__) || defined (__cplusplus)))
-void free (void *); /* INFRINGES ON USER NAME SPACE */
-#   endif
-#  endif
-#  ifdef __cplusplus
-}
-#  endif
 # endif
 #endif /* ! defined (yyoverflow) || YYERROR_VERBOSE */
 
@@ -376,7 +322,7 @@ void free (void *); /* INFRINGES ON USER NAME SPACE */
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  short int yyss;
+  short yyss;
   YYSTYPE yyvs;
   };
 
@@ -386,7 +332,7 @@ union yyalloc
 /* The size of an array large to enough to hold all stacks, each with
    N elements.  */
 # define YYSTACK_BYTES(N) \
-     ((N) * (sizeof (short int) + sizeof (YYSTYPE))			\
+     ((N) * (sizeof (short) + sizeof (YYSTYPE))				\
       + YYSTACK_GAP_MAXIMUM)
 
 /* Copy COUNT objects from FROM to TO.  The source and destination do
@@ -399,7 +345,7 @@ union yyalloc
 #   define YYCOPY(To, From, Count)		\
       do					\
 	{					\
-	  YYSIZE_T yyi;				\
+	  register YYSIZE_T yyi;		\
 	  for (yyi = 0; yyi < (Count); yyi++)	\
 	    (To)[yyi] = (From)[yyi];		\
 	}					\
@@ -428,7 +374,7 @@ union yyalloc
 #if defined (__STDC__) || defined (__cplusplus)
    typedef signed char yysigned_char;
 #else
-   typedef short int yysigned_char;
+   typedef short yysigned_char;
 #endif
 
 /* YYFINAL -- State number of the termination state. */
@@ -449,7 +395,7 @@ union yyalloc
 #define YYUNDEFTOK  2
 #define YYMAXUTOK   292
 
-#define YYTRANSLATE(YYX)						\
+#define YYTRANSLATE(YYX) 						\
   ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
 
 /* YYTRANSLATE[YYLEX] -- Bison symbol number corresponding to YYLEX.  */
@@ -520,7 +466,7 @@ static const yysigned_char yyrhs[] =
 };
 
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
-static const unsigned short int yyrline[] =
+static const unsigned short yyrline[] =
 {
        0,   144,   144,   149,   151,   157,   161,   166,   183,   187,
      205,   209,   225,   230,   229,   237,   242,   247,   252,   257,
@@ -531,8 +477,8 @@ static const unsigned short int yyrline[] =
 };
 #endif
 
-#if YYDEBUG || YYERROR_VERBOSE || YYTOKEN_TABLE
-/* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
+#if YYDEBUG || YYERROR_VERBOSE
+/* YYTNME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
    First, the terminals, then, starting at YYNTOKENS, nonterminals. */
 static const char *const yytname[] =
 {
@@ -551,7 +497,7 @@ static const char *const yytname[] =
 # ifdef YYPRINT
 /* YYTOKNUM[YYLEX-NUM] -- Internal token number corresponding to
    token YYLEX-NUM.  */
-static const unsigned short int yytoknum[] =
+static const unsigned short yytoknum[] =
 {
        0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
      265,   266,   267,   268,   269,   270,   271,   272,   273,   274,
@@ -672,6 +618,22 @@ static const unsigned char yystos[] =
       36,    39
 };
 
+#if ! defined (YYSIZE_T) && defined (__SIZE_TYPE__)
+# define YYSIZE_T __SIZE_TYPE__
+#endif
+#if ! defined (YYSIZE_T) && defined (size_t)
+# define YYSIZE_T size_t
+#endif
+#if ! defined (YYSIZE_T)
+# if defined (__STDC__) || defined (__cplusplus)
+#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
+#  define YYSIZE_T size_t
+# endif
+#endif
+#if ! defined (YYSIZE_T)
+# define YYSIZE_T unsigned int
+#endif
+
 #define yyerrok		(yyerrstatus = 0)
 #define yyclearin	(yychar = YYEMPTY)
 #define YYEMPTY		(-2)
@@ -701,59 +663,26 @@ do								\
       goto yybackup;						\
     }								\
   else								\
-    {								\
-      yyerror (YY_("syntax error: cannot back up")); \
+    { 								\
+      yyerror ("syntax error: cannot back up");\
       YYERROR;							\
     }								\
 while (0)
 
-
 #define YYTERROR	1
 #define YYERRCODE	256
 
+/* YYLLOC_DEFAULT -- Compute the default location (before the actions
+   are run).  */
 
-/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
-   If N is 0, then set CURRENT to the empty location which ends
-   the previous symbol: RHS[0] (always defined).  */
-
-#define YYRHSLOC(Rhs, K) ((Rhs)[K])
 #ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)				\
-    do									\
-      if (N)								\
-	{								\
-	  (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;	\
-	  (Current).first_column = YYRHSLOC (Rhs, 1).first_column;	\
-	  (Current).last_line    = YYRHSLOC (Rhs, N).last_line;		\
-	  (Current).last_column  = YYRHSLOC (Rhs, N).last_column;	\
-	}								\
-      else								\
-	{								\
-	  (Current).first_line   = (Current).last_line   =		\
-	    YYRHSLOC (Rhs, 0).last_line;				\
-	  (Current).first_column = (Current).last_column =		\
-	    YYRHSLOC (Rhs, 0).last_column;				\
-	}								\
-    while (0)
+# define YYLLOC_DEFAULT(Current, Rhs, N)		\
+   ((Current).first_line   = (Rhs)[1].first_line,	\
+    (Current).first_column = (Rhs)[1].first_column,	\
+    (Current).last_line    = (Rhs)[N].last_line,	\
+    (Current).last_column  = (Rhs)[N].last_column)
 #endif
 
-
-/* YY_LOCATION_PRINT -- Print the location on the stream.
-   This macro was not mandated originally: define only if we know
-   we won't break user code: when these are the locations we know.  */
-
-#ifndef YY_LOCATION_PRINT
-# if YYLTYPE_IS_TRIVIAL
-#  define YY_LOCATION_PRINT(File, Loc)			\
-     fprintf (File, "%d.%d-%d.%d",			\
-              (Loc).first_line, (Loc).first_column,	\
-              (Loc).last_line,  (Loc).last_column)
-# else
-#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
-# endif
-#endif
-
-
 /* YYLEX -- calling `yylex' with the right arguments.  */
 
 #ifdef YYLEX_PARAM
@@ -776,13 +705,19 @@ do {						\
     YYFPRINTF Args;				\
 } while (0)
 
-# define YY_SYMBOL_PRINT(Title, Type, Value, Location)		\
+# define YYDSYMPRINT(Args)			\
+do {						\
+  if (yydebug)					\
+    yysymprint Args;				\
+} while (0)
+
+# define YYDSYMPRINTF(Title, Token, Value, Location)		\
 do {								\
   if (yydebug)							\
     {								\
       YYFPRINTF (stderr, "%s ", Title);				\
-      yysymprint (stderr,					\
-                  Type, Value);	\
+      yysymprint (stderr, 					\
+                  Token, Value);	\
       YYFPRINTF (stderr, "\n");					\
     }								\
 } while (0)
@@ -794,12 +729,12 @@ do {								\
 
 #if defined (__STDC__) || defined (__cplusplus)
 static void
-yy_stack_print (short int *bottom, short int *top)
+yy_stack_print (short *bottom, short *top)
 #else
 static void
 yy_stack_print (bottom, top)
-    short int *bottom;
-    short int *top;
+    short *bottom;
+    short *top;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
@@ -829,13 +764,13 @@ yy_reduce_print (yyrule)
 #endif
 {
   int yyi;
-  unsigned long int yylno = yyrline[yyrule];
-  YYFPRINTF (stderr, "Reducing stack by rule %d (line %lu), ",
+  unsigned int yylno = yyrline[yyrule];
+  YYFPRINTF (stderr, "Reducing stack by rule %d (line %u), ",
              yyrule - 1, yylno);
   /* Print the symbols being reduced, and their result.  */
   for (yyi = yyprhs[yyrule]; 0 <= yyrhs[yyi]; yyi++)
-    YYFPRINTF (stderr, "%s ", yytname[yyrhs[yyi]]);
-  YYFPRINTF (stderr, "-> %s\n", yytname[yyr1[yyrule]]);
+    YYFPRINTF (stderr, "%s ", yytname [yyrhs[yyi]]);
+  YYFPRINTF (stderr, "-> %s\n", yytname [yyr1[yyrule]]);
 }
 
 # define YY_REDUCE_PRINT(Rule)		\
@@ -849,7 +784,8 @@ do {					\
 int yydebug;
 #else /* !YYDEBUG */
 # define YYDPRINTF(Args)
-# define YY_SYMBOL_PRINT(Title, Type, Value, Location)
+# define YYDSYMPRINT(Args)
+# define YYDSYMPRINTF(Title, Token, Value, Location)
 # define YY_STACK_PRINT(Bottom, Top)
 # define YY_REDUCE_PRINT(Rule)
 #endif /* !YYDEBUG */
@@ -864,9 +800,13 @@ int yydebug;
    if the built-in stack extension method is used).
 
    Do not make this value too large; the results are undefined if
-   YYSTACK_ALLOC_MAXIMUM < YYSTACK_BYTES (YYMAXDEPTH)
+   SIZE_MAX < YYSTACK_BYTES (YYMAXDEPTH)
    evaluated with infinite-precision integer arithmetic.  */
 
+#if defined (YYMAXDEPTH) && YYMAXDEPTH == 0
+# undef YYMAXDEPTH
+#endif
+
 #ifndef YYMAXDEPTH
 # define YYMAXDEPTH 10000
 #endif
@@ -888,7 +828,7 @@ yystrlen (yystr)
      const char *yystr;
 #   endif
 {
-  const char *yys = yystr;
+  register const char *yys = yystr;
 
   while (*yys++ != '\0')
     continue;
@@ -913,8 +853,8 @@ yystpcpy (yydest, yysrc)
      const char *yysrc;
 #   endif
 {
-  char *yyd = yydest;
-  const char *yys = yysrc;
+  register char *yyd = yydest;
+  register const char *yys = yysrc;
 
   while ((*yyd++ = *yys++) != '\0')
     continue;
@@ -924,55 +864,7 @@ yystpcpy (yydest, yysrc)
 #  endif
 # endif
 
-# ifndef yytnamerr
-/* Copy to YYRES the contents of YYSTR after stripping away unnecessary
-   quotes and backslashes, so that it's suitable for yyerror.  The
-   heuristic is that double-quoting is unnecessary unless the string
-   contains an apostrophe, a comma, or backslash (other than
-   backslash-backslash).  YYSTR is taken from yytname.  If YYRES is
-   null, do not copy; instead, return the length of what the result
-   would have been.  */
-static YYSIZE_T
-yytnamerr (char *yyres, const char *yystr)
-{
-  if (*yystr == '"')
-    {
-      size_t yyn = 0;
-      char const *yyp = yystr;
-
-      for (;;)
-	switch (*++yyp)
-	  {
-	  case '\'':
-	  case ',':
-	    goto do_not_strip_quotes;
-
-	  case '\\':
-	    if (*++yyp != '\\')
-	      goto do_not_strip_quotes;
-	    /* Fall through.  */
-	  default:
-	    if (yyres)
-	      yyres[yyn] = *yyp;
-	    yyn++;
-	    break;
-
-	  case '"':
-	    if (yyres)
-	      yyres[yyn] = '\0';
-	    return yyn;
-	  }
-    do_not_strip_quotes: ;
-    }
-
-  if (! yyres)
-    return yystrlen (yystr);
-
-  return yystpcpy (yyres, yystr) - yyres;
-}
-# endif
-
-#endif /* YYERROR_VERBOSE */
+#endif /* !YYERROR_VERBOSE */
 
 
 
@@ -996,15 +888,15 @@ yysymprint (yyoutput, yytype, yyvaluep)
   (void) yyvaluep;
 
   if (yytype < YYNTOKENS)
-    YYFPRINTF (yyoutput, "token %s (", yytname[yytype]);
+    {
+      YYFPRINTF (yyoutput, "token %s (", yytname[yytype]);
+# ifdef YYPRINT
+      YYPRINT (yyoutput, yytoknum[yytype], *yyvaluep);
+# endif
+    }
   else
     YYFPRINTF (yyoutput, "nterm %s (", yytname[yytype]);
 
-
-# ifdef YYPRINT
-  if (yytype < YYNTOKENS)
-    YYPRINT (yyoutput, yytoknum[yytype], *yyvaluep);
-# endif
   switch (yytype)
     {
       default:
@@ -1020,11 +912,10 @@ yysymprint (yyoutput, yytype, yyvaluep)
 
 #if defined (__STDC__) || defined (__cplusplus)
 static void
-yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
+yydestruct (int yytype, YYSTYPE *yyvaluep)
 #else
 static void
-yydestruct (yymsg, yytype, yyvaluep)
-    const char *yymsg;
+yydestruct (yytype, yyvaluep)
     int yytype;
     YYSTYPE *yyvaluep;
 #endif
@@ -1032,10 +923,6 @@ yydestruct (yymsg, yytype, yyvaluep)
   /* Pacify ``unused variable'' warnings.  */
   (void) yyvaluep;
 
-  if (!yymsg)
-    yymsg = "Deleting";
-  YY_SYMBOL_PRINT (yymsg, yytype, yyvaluep, yylocationp);
-
   switch (yytype)
     {
 
@@ -1063,10 +950,10 @@ int yyparse ();
 
 
 
-/* The look-ahead symbol.  */
+/* The lookahead symbol.  */
 int yychar;
 
-/* The semantic value of the look-ahead symbol.  */
+/* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
 /* Number of syntax errors so far.  */
@@ -1092,17 +979,17 @@ yyparse (void)
 #else
 int
 yyparse ()
-    ;
+
 #endif
 #endif
 {
   
-  int yystate;
-  int yyn;
+  register int yystate;
+  register int yyn;
   int yyresult;
   /* Number of tokens to shift before error messages enabled.  */
   int yyerrstatus;
-  /* Look-ahead token as an internal (translated) token number.  */
+  /* Lookahead token as an internal (translated) token number.  */
   int yytoken = 0;
 
   /* Three stacks and their tools:
@@ -1114,14 +1001,14 @@ yyparse ()
      to reallocate them elsewhere.  */
 
   /* The state stack.  */
-  short int yyssa[YYINITDEPTH];
-  short int *yyss = yyssa;
-  short int *yyssp;
+  short	yyssa[YYINITDEPTH];
+  short *yyss = yyssa;
+  register short *yyssp;
 
   /* The semantic value stack.  */
   YYSTYPE yyvsa[YYINITDEPTH];
   YYSTYPE *yyvs = yyvsa;
-  YYSTYPE *yyvsp;
+  register YYSTYPE *yyvsp;
 
 
 
@@ -1178,14 +1065,14 @@ yyparse ()
 	   these so that the &'s don't force the real ones into
 	   memory.  */
 	YYSTYPE *yyvs1 = yyvs;
-	short int *yyss1 = yyss;
+	short *yyss1 = yyss;
 
 
 	/* Each stack pointer address is followed by the size of the
 	   data in use in that stack, in bytes.  This used to be a
 	   conditional around just the two extra args, but that might
 	   be undefined if yyoverflow is a macro.  */
-	yyoverflow (YY_("memory exhausted"),
+	yyoverflow ("parser stack overflow",
 		    &yyss1, yysize * sizeof (*yyssp),
 		    &yyvs1, yysize * sizeof (*yyvsp),
 
@@ -1196,21 +1083,21 @@ yyparse ()
       }
 #else /* no yyoverflow */
 # ifndef YYSTACK_RELOCATE
-      goto yyexhaustedlab;
+      goto yyoverflowlab;
 # else
       /* Extend the stack our own way.  */
       if (YYMAXDEPTH <= yystacksize)
-	goto yyexhaustedlab;
+	goto yyoverflowlab;
       yystacksize *= 2;
       if (YYMAXDEPTH < yystacksize)
 	yystacksize = YYMAXDEPTH;
 
       {
-	short int *yyss1 = yyss;
+	short *yyss1 = yyss;
 	union yyalloc *yyptr =
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
-	  goto yyexhaustedlab;
+	  goto yyoverflowlab;
 	YYSTACK_RELOCATE (yyss);
 	YYSTACK_RELOCATE (yyvs);
 
@@ -1242,18 +1129,18 @@ yyparse ()
 yybackup:
 
 /* Do appropriate processing given the current state.  */
-/* Read a look-ahead token if we need one and don't already have one.  */
+/* Read a lookahead token if we need one and don't already have one.  */
 /* yyresume: */
 
-  /* First try to decide what to do without reference to look-ahead token.  */
+  /* First try to decide what to do without reference to lookahead token.  */
 
   yyn = yypact[yystate];
   if (yyn == YYPACT_NINF)
     goto yydefault;
 
-  /* Not known => get a look-ahead token if don't already have one.  */
+  /* Not known => get a lookahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1268,7 +1155,7 @@ yybackup:
   else
     {
       yytoken = YYTRANSLATE (yychar);
-      YY_SYMBOL_PRINT ("Next token is", yytoken, &yylval, &yylloc);
+      YYDSYMPRINTF ("Next token is", yytoken, &yylval, &yylloc);
     }
 
   /* If the proper action on seeing token YYTOKEN is to reduce or to
@@ -1288,8 +1175,8 @@ yybackup:
   if (yyn == YYFINAL)
     YYACCEPT;
 
-  /* Shift the look-ahead token.  */
-  YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
+  /* Shift the lookahead token.  */
+  YYDPRINTF ((stderr, "Shifting token %s, ", yytname[yytoken]));
 
   /* Discard the token being shifted unless it is eof.  */
   if (yychar != YYEOF)
@@ -1341,7 +1228,7 @@ yyreduce:
         case 5:
 #line 158 "nlmheader.y"
     {
-	    check_procedure = (yyvsp[0].string);
+	    check_procedure = yyvsp[0].string;
 	  }
     break;
 
@@ -1349,7 +1236,7 @@ yyreduce:
 #line 162 "nlmheader.y"
     {
 	    nlmheader_warn (_("CODESTART is not implemented; sorry"), -1);
-	    free ((yyvsp[0].string));
+	    free (yyvsp[0].string);
 	  }
     break;
 
@@ -1359,7 +1246,7 @@ yyreduce:
 	    int len;
 
 	    strncpy (copyright_hdr->stamp, "CoPyRiGhT=", 10);
-	    len = strlen ((yyvsp[0].string));
+	    len = strlen (yyvsp[0].string);
 	    if (len >= NLM_MAX_COPYRIGHT_MESSAGE_LENGTH)
 	      {
 		nlmheader_warn (_("copyright string is too long"),
@@ -1367,16 +1254,16 @@ yyreduce:
 		len = NLM_MAX_COPYRIGHT_MESSAGE_LENGTH - 1;
 	      }
 	    copyright_hdr->copyrightMessageLength = len;
-	    strncpy (copyright_hdr->copyrightMessage, (yyvsp[0].string), len);
+	    strncpy (copyright_hdr->copyrightMessage, yyvsp[0].string, len);
 	    copyright_hdr->copyrightMessage[len] = '\0';
-	    free ((yyvsp[0].string));
+	    free (yyvsp[0].string);
 	  }
     break;
 
   case 8:
 #line 184 "nlmheader.y"
     {
-	    custom_file = (yyvsp[0].string);
+	    custom_file = yyvsp[0].string;
 	  }
     break;
 
@@ -1386,12 +1273,12 @@ yyreduce:
 	    /* We don't set the version stamp here, because we use the
 	       version stamp to detect whether the required VERSION
 	       keyword was given.  */
-	    version_hdr->month = nlmlex_get_number ((yyvsp[-2].string));
-	    version_hdr->day = nlmlex_get_number ((yyvsp[-1].string));
-	    version_hdr->year = nlmlex_get_number ((yyvsp[0].string));
-	    free ((yyvsp[-2].string));
-	    free ((yyvsp[-1].string));
-	    free ((yyvsp[0].string));
+	    version_hdr->month = nlmlex_get_number (yyvsp[-2].string);
+	    version_hdr->day = nlmlex_get_number (yyvsp[-1].string);
+	    version_hdr->year = nlmlex_get_number (yyvsp[0].string);
+	    free (yyvsp[-2].string);
+	    free (yyvsp[-1].string);
+	    free (yyvsp[0].string);
 	    if (version_hdr->month < 1 || version_hdr->month > 12)
 	      nlmheader_warn (_("illegal month"), -1);
 	    if (version_hdr->day < 1 || version_hdr->day > 31)
@@ -1413,7 +1300,7 @@ yyreduce:
     {
 	    int len;
 
-	    len = strlen ((yyvsp[0].string));
+	    len = strlen (yyvsp[0].string);
 	    if (len > NLM_MAX_DESCRIPTION_LENGTH)
 	      {
 		nlmheader_warn (_("description string is too long"),
@@ -1421,16 +1308,16 @@ yyreduce:
 		len = NLM_MAX_DESCRIPTION_LENGTH;
 	      }
 	    var_hdr->descriptionLength = len;
-	    strncpy (var_hdr->descriptionText, (yyvsp[0].string), len);
+	    strncpy (var_hdr->descriptionText, yyvsp[0].string, len);
 	    var_hdr->descriptionText[len] = '\0';
-	    free ((yyvsp[0].string));
+	    free (yyvsp[0].string);
 	  }
     break;
 
   case 12:
 #line 226 "nlmheader.y"
     {
-	    exit_procedure = (yyvsp[0].string);
+	    exit_procedure = yyvsp[0].string;
 	  }
     break;
 
@@ -1444,23 +1331,23 @@ yyreduce:
   case 14:
 #line 234 "nlmheader.y"
     {
-	    export_symbols = string_list_append (export_symbols, (yyvsp[0].list));
+	    export_symbols = string_list_append (export_symbols, yyvsp[0].list);
 	  }
     break;
 
   case 15:
 #line 238 "nlmheader.y"
     {
-	    fixed_hdr->flags |= nlmlex_get_number ((yyvsp[0].string));
-	    free ((yyvsp[0].string));
+	    fixed_hdr->flags |= nlmlex_get_number (yyvsp[0].string);
+	    free (yyvsp[0].string);
 	  }
     break;
 
   case 16:
 #line 243 "nlmheader.y"
     {
-	    fixed_hdr->flags &=~ nlmlex_get_number ((yyvsp[0].string));
-	    free ((yyvsp[0].string));
+	    fixed_hdr->flags &=~ nlmlex_get_number (yyvsp[0].string);
+	    free (yyvsp[0].string);
 	  }
     break;
 
@@ -1475,7 +1362,7 @@ yyreduce:
   case 18:
 #line 253 "nlmheader.y"
     {
-	    map_file = (yyvsp[0].string);
+	    map_file = yyvsp[0].string;
 	    full_map = TRUE;
 	  }
     break;
@@ -1483,7 +1370,7 @@ yyreduce:
   case 19:
 #line 258 "nlmheader.y"
     {
-	    help_file = (yyvsp[0].string);
+	    help_file = yyvsp[0].string;
 	  }
     break;
 
@@ -1497,14 +1384,14 @@ yyreduce:
   case 21:
 #line 266 "nlmheader.y"
     {
-	    import_symbols = string_list_append (import_symbols, (yyvsp[0].list));
+	    import_symbols = string_list_append (import_symbols, yyvsp[0].list);
 	  }
     break;
 
   case 22:
 #line 270 "nlmheader.y"
     {
-	    input_files = string_list_append (input_files, (yyvsp[0].list));
+	    input_files = string_list_append (input_files, yyvsp[0].list);
 	  }
     break;
 
@@ -1518,21 +1405,21 @@ yyreduce:
   case 24:
 #line 278 "nlmheader.y"
     {
-	    map_file = (yyvsp[0].string);
+	    map_file = yyvsp[0].string;
 	  }
     break;
 
   case 25:
 #line 282 "nlmheader.y"
     {
-	    message_file = (yyvsp[0].string);
+	    message_file = yyvsp[0].string;
 	  }
     break;
 
   case 26:
 #line 286 "nlmheader.y"
     {
-	    modules = string_list_append (modules, (yyvsp[0].list));
+	    modules = string_list_append (modules, yyvsp[0].list);
 	  }
     break;
 
@@ -1554,7 +1441,7 @@ yyreduce:
 #line 298 "nlmheader.y"
     {
 	    if (output_file == NULL)
-	      output_file = (yyvsp[0].string);
+	      output_file = yyvsp[0].string;
 	    else
 	      nlmheader_warn (_("ignoring duplicate OUTPUT statement"), -1);
 	  }
@@ -1579,7 +1466,7 @@ yyreduce:
     {
 	    int len;
 
-	    len = strlen ((yyvsp[0].string));
+	    len = strlen (yyvsp[0].string);
 	    if (len >= NLM_MAX_SCREEN_NAME_LENGTH)
 	      {
 		nlmheader_warn (_("screen name is too long"),
@@ -1587,31 +1474,31 @@ yyreduce:
 		len = NLM_MAX_SCREEN_NAME_LENGTH;
 	      }
 	    var_hdr->screenNameLength = len;
-	    strncpy (var_hdr->screenName, (yyvsp[0].string), len);
+	    strncpy (var_hdr->screenName, yyvsp[0].string, len);
 	    var_hdr->screenName[NLM_MAX_SCREEN_NAME_LENGTH] = '\0';
-	    free ((yyvsp[0].string));
+	    free (yyvsp[0].string);
 	  }
     break;
 
   case 33:
 #line 329 "nlmheader.y"
     {
-	    sharelib_file = (yyvsp[0].string);
+	    sharelib_file = yyvsp[0].string;
 	  }
     break;
 
   case 34:
 #line 333 "nlmheader.y"
     {
-	    var_hdr->stackSize = nlmlex_get_number ((yyvsp[0].string));
-	    free ((yyvsp[0].string));
+	    var_hdr->stackSize = nlmlex_get_number (yyvsp[0].string);
+	    free (yyvsp[0].string);
 	  }
     break;
 
   case 35:
 #line 338 "nlmheader.y"
     {
-	    start_procedure = (yyvsp[0].string);
+	    start_procedure = yyvsp[0].string;
 	  }
     break;
 
@@ -1627,7 +1514,7 @@ yyreduce:
     {
 	    int len;
 
-	    len = strlen ((yyvsp[0].string));
+	    len = strlen (yyvsp[0].string);
 	    if (len >= NLM_MAX_THREAD_NAME_LENGTH)
 	      {
 		nlmheader_warn (_("thread name is too long"),
@@ -1635,17 +1522,17 @@ yyreduce:
 		len = NLM_MAX_THREAD_NAME_LENGTH;
 	      }
 	    var_hdr->threadNameLength = len;
-	    strncpy (var_hdr->threadName, (yyvsp[0].string), len);
+	    strncpy (var_hdr->threadName, yyvsp[0].string, len);
 	    var_hdr->threadName[len] = '\0';
-	    free ((yyvsp[0].string));
+	    free (yyvsp[0].string);
 	  }
     break;
 
   case 38:
 #line 362 "nlmheader.y"
     {
-	    fixed_hdr->moduleType = nlmlex_get_number ((yyvsp[0].string));
-	    free ((yyvsp[0].string));
+	    fixed_hdr->moduleType = nlmlex_get_number (yyvsp[0].string);
+	    free (yyvsp[0].string);
 	  }
     break;
 
@@ -1662,14 +1549,14 @@ yyreduce:
 	    long val;
 
 	    strncpy (version_hdr->stamp, "VeRsIoN#", 8);
-	    version_hdr->majorVersion = nlmlex_get_number ((yyvsp[-2].string));
-	    val = nlmlex_get_number ((yyvsp[-1].string));
+	    version_hdr->majorVersion = nlmlex_get_number (yyvsp[-2].string);
+	    val = nlmlex_get_number (yyvsp[-1].string);
 	    if (val < 0 || val > 99)
 	      nlmheader_warn (_("illegal minor version number (must be between 0 and 99)"),
 			      -1);
 	    else
 	      version_hdr->minorVersion = val;
-	    val = nlmlex_get_number ((yyvsp[0].string));
+	    val = nlmlex_get_number (yyvsp[0].string);
 	    if (val < 0)
 	      nlmheader_warn (_("illegal revision number (must be between 0 and 26)"),
 			      -1);
@@ -1677,9 +1564,9 @@ yyreduce:
 	      version_hdr->revision = 0;
 	    else
 	      version_hdr->revision = val;
-	    free ((yyvsp[-2].string));
-	    free ((yyvsp[-1].string));
-	    free ((yyvsp[0].string));
+	    free (yyvsp[-2].string);
+	    free (yyvsp[-1].string);
+	    free (yyvsp[0].string);
 	  }
     break;
 
@@ -1689,65 +1576,65 @@ yyreduce:
 	    long val;
 
 	    strncpy (version_hdr->stamp, "VeRsIoN#", 8);
-	    version_hdr->majorVersion = nlmlex_get_number ((yyvsp[-1].string));
-	    val = nlmlex_get_number ((yyvsp[0].string));
+	    version_hdr->majorVersion = nlmlex_get_number (yyvsp[-1].string);
+	    val = nlmlex_get_number (yyvsp[0].string);
 	    if (val < 0 || val > 99)
 	      nlmheader_warn (_("illegal minor version number (must be between 0 and 99)"),
 			      -1);
 	    else
 	      version_hdr->minorVersion = val;
 	    version_hdr->revision = 0;
-	    free ((yyvsp[-1].string));
-	    free ((yyvsp[0].string));
+	    free (yyvsp[-1].string);
+	    free (yyvsp[0].string);
 	  }
     break;
 
   case 42:
 #line 411 "nlmheader.y"
     {
-	    rpc_file = (yyvsp[0].string);
+	    rpc_file = yyvsp[0].string;
 	  }
     break;
 
   case 43:
 #line 420 "nlmheader.y"
     {
-	    (yyval.list) = NULL;
+	    yyval.list = NULL;
 	  }
     break;
 
   case 44:
 #line 424 "nlmheader.y"
     {
-	    (yyval.list) = (yyvsp[0].list);
+	    yyval.list = yyvsp[0].list;
 	  }
     break;
 
   case 45:
 #line 435 "nlmheader.y"
     {
-	    (yyval.list) = string_list_cons ((yyvsp[0].string), NULL);
+	    yyval.list = string_list_cons (yyvsp[0].string, NULL);
 	  }
     break;
 
   case 46:
 #line 439 "nlmheader.y"
     {
-	    (yyval.list) = NULL;
+	    yyval.list = NULL;
 	  }
     break;
 
   case 47:
 #line 443 "nlmheader.y"
     {
-	    (yyval.list) = string_list_append1 ((yyvsp[-1].list), (yyvsp[0].string));
+	    yyval.list = string_list_append1 (yyvsp[-1].list, yyvsp[0].string);
 	  }
     break;
 
   case 48:
 #line 447 "nlmheader.y"
     {
-	    (yyval.list) = (yyvsp[-1].list);
+	    yyval.list = yyvsp[-1].list;
 	  }
     break;
 
@@ -1756,7 +1643,7 @@ yyreduce:
     {
 	    if (symbol_prefix != NULL)
 	      free (symbol_prefix);
-	    symbol_prefix = (yyvsp[-1].string);
+	    symbol_prefix = yyvsp[-1].string;
 	  }
     break;
 
@@ -1764,12 +1651,12 @@ yyreduce:
 #line 467 "nlmheader.y"
     {
 	    if (symbol_prefix == NULL)
-	      (yyval.string) = (yyvsp[0].string);
+	      yyval.string = yyvsp[0].string;
 	    else
 	      {
-		(yyval.string) = xmalloc (strlen (symbol_prefix) + strlen ((yyvsp[0].string)) + 2);
-		sprintf ((yyval.string), "%s@%s", symbol_prefix, (yyvsp[0].string));
-		free ((yyvsp[0].string));
+		yyval.string = xmalloc (strlen (symbol_prefix) + strlen (yyvsp[0].string) + 2);
+		sprintf (yyval.string, "%s@%s", symbol_prefix, yyvsp[0].string);
+		free (yyvsp[0].string);
 	      }
 	  }
     break;
@@ -1777,23 +1664,22 @@ yyreduce:
   case 51:
 #line 483 "nlmheader.y"
     {
-	    (yyval.list) = NULL;
+	    yyval.list = NULL;
 	  }
     break;
 
   case 52:
 #line 487 "nlmheader.y"
     {
-	    (yyval.list) = string_list_cons ((yyvsp[-1].string), (yyvsp[0].list));
+	    yyval.list = string_list_cons (yyvsp[-1].string, yyvsp[0].list);
 	  }
     break;
 
 
-      default: break;
     }
 
-/* Line 1126 of yacc.c.  */
-#line 1797 "nlmheader.c"
+/* Line 1000 of yacc.c.  */
+#line 1683 "nlmheader.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1832,36 +1718,12 @@ yyerrlab:
 
       if (YYPACT_NINF < yyn && yyn < YYLAST)
 	{
+	  YYSIZE_T yysize = 0;
 	  int yytype = YYTRANSLATE (yychar);
-	  YYSIZE_T yysize0 = yytnamerr (0, yytname[yytype]);
-	  YYSIZE_T yysize = yysize0;
-	  YYSIZE_T yysize1;
-	  int yysize_overflow = 0;
-	  char *yymsg = 0;
-#	  define YYERROR_VERBOSE_ARGS_MAXIMUM 5
-	  char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
+	  const char* yyprefix;
+	  char *yymsg;
 	  int yyx;
 
-#if 0
-	  /* This is so xgettext sees the translatable formats that are
-	     constructed on the fly.  */
-	  YY_("syntax error, unexpected %s");
-	  YY_("syntax error, unexpected %s, expecting %s");
-	  YY_("syntax error, unexpected %s, expecting %s or %s");
-	  YY_("syntax error, unexpected %s, expecting %s or %s or %s");
-	  YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s");
-#endif
-	  char *yyfmt;
-	  char const *yyf;
-	  static char const yyunexpected[] = "syntax error, unexpected %s";
-	  static char const yyexpecting[] = ", expecting %s";
-	  static char const yyor[] = " or %s";
-	  char yyformat[sizeof yyunexpected
-			+ sizeof yyexpecting - 1
-			+ ((YYERROR_VERBOSE_ARGS_MAXIMUM - 2)
-			   * (sizeof yyor - 1))];
-	  char const *yyprefix = yyexpecting;
-
 	  /* Start YYX at -YYN if negative to avoid negative indexes in
 	     YYCHECK.  */
 	  int yyxbegin = yyn < 0 ? -yyn : 0;
@@ -1869,91 +1731,81 @@ yyerrlab:
 	  /* Stay within bounds of both yycheck and yytname.  */
 	  int yychecklim = YYLAST - yyn;
 	  int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
-	  int yycount = 1;
-
-	  yyarg[0] = yytname[yytype];
-	  yyfmt = yystpcpy (yyformat, yyunexpected);
+	  int yycount = 0;
 
+	  yyprefix = ", expecting ";
 	  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
 	    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
 	      {
-		if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
+		yysize += yystrlen (yyprefix) + yystrlen (yytname [yyx]);
+		yycount += 1;
+		if (yycount == 5)
 		  {
-		    yycount = 1;
-		    yysize = yysize0;
-		    yyformat[sizeof yyunexpected - 1] = '\0';
+		    yysize = 0;
 		    break;
 		  }
-		yyarg[yycount++] = yytname[yyx];
-		yysize1 = yysize + yytnamerr (0, yytname[yyx]);
-		yysize_overflow |= yysize1 < yysize;
-		yysize = yysize1;
-		yyfmt = yystpcpy (yyfmt, yyprefix);
-		yyprefix = yyor;
 	      }
-
-	  yyf = YY_(yyformat);
-	  yysize1 = yysize + yystrlen (yyf);
-	  yysize_overflow |= yysize1 < yysize;
-	  yysize = yysize1;
-
-	  if (!yysize_overflow && yysize <= YYSTACK_ALLOC_MAXIMUM)
-	    yymsg = (char *) YYSTACK_ALLOC (yysize);
-	  if (yymsg)
+	  yysize += (sizeof ("syntax error, unexpected ")
+		     + yystrlen (yytname[yytype]));
+	  yymsg = (char *) YYSTACK_ALLOC (yysize);
+	  if (yymsg != 0)
 	    {
-	      /* Avoid sprintf, as that infringes on the user's name space.
-		 Don't have undefined behavior even if the translation
-		 produced a string with the wrong number of "%s"s.  */
-	      char *yyp = yymsg;
-	      int yyi = 0;
-	      while ((*yyp = *yyf))
+	      char *yyp = yystpcpy (yymsg, "syntax error, unexpected ");
+	      yyp = yystpcpy (yyp, yytname[yytype]);
+
+	      if (yycount < 5)
 		{
-		  if (*yyp == '%' && yyf[1] == 's' && yyi < yycount)
-		    {
-		      yyp += yytnamerr (yyp, yyarg[yyi++]);
-		      yyf += 2;
-		    }
-		  else
-		    {
-		      yyp++;
-		      yyf++;
-		    }
+		  yyprefix = ", expecting ";
+		  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
+		    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
+		      {
+			yyp = yystpcpy (yyp, yyprefix);
+			yyp = yystpcpy (yyp, yytname[yyx]);
+			yyprefix = " or ";
+		      }
 		}
 	      yyerror (yymsg);
 	      YYSTACK_FREE (yymsg);
 	    }
 	  else
-	    {
-	      yyerror (YY_("syntax error"));
-	      goto yyexhaustedlab;
-	    }
+	    yyerror ("syntax error; also virtual memory exhausted");
 	}
       else
 #endif /* YYERROR_VERBOSE */
-	yyerror (YY_("syntax error"));
+	yyerror ("syntax error");
     }
 
 
 
   if (yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse look-ahead token after an
+      /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
       if (yychar <= YYEOF)
         {
-	  /* Return failure if at end of input.  */
+          /* If at end of input, pop the error token,
+	     then the rest of the stack, then return failure.  */
 	  if (yychar == YYEOF)
-	    YYABORT;
+	     for (;;)
+	       {
+		 YYPOPSTACK;
+		 if (yyssp == yyss)
+		   YYABORT;
+		 YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+		 yydestruct (yystos[*yyssp], yyvsp);
+	       }
         }
       else
 	{
-	  yydestruct ("Error: discarding", yytoken, &yylval);
+	  YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
+	  yydestruct (yytoken, &yylval);
 	  yychar = YYEMPTY;
+
 	}
     }
 
-  /* Else will try to reuse look-ahead token after shifting the error
+  /* Else will try to reuse lookahead token after shifting the error
      token.  */
   goto yyerrlab1;
 
@@ -1963,13 +1815,14 @@ yyerrlab:
 `---------------------------------------------------*/
 yyerrorlab:
 
-  /* Pacify compilers like GCC when the user code never invokes
-     YYERROR and the label yyerrorlab therefore never appears in user
-     code.  */
+#ifdef __GNUC__
+  /* Pacify GCC when the user code never invokes YYERROR and the label
+     yyerrorlab therefore never appears in user code.  */
   if (0)
      goto yyerrorlab;
+#endif
 
-yyvsp -= yylen;
+  yyvsp -= yylen;
   yyssp -= yylen;
   yystate = *yyssp;
   goto yyerrlab1;
@@ -1999,8 +1852,8 @@ yyerrlab1:
       if (yyssp == yyss)
 	YYABORT;
 
-
-      yydestruct ("Error: popping", yystos[yystate], yyvsp);
+      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+      yydestruct (yystos[yystate], yyvsp);
       YYPOPSTACK;
       yystate = *yyssp;
       YY_STACK_PRINT (yyss, yyssp);
@@ -2009,12 +1862,11 @@ yyerrlab1:
   if (yyn == YYFINAL)
     YYACCEPT;
 
+  YYDPRINTF ((stderr, "Shifting error token, "));
+
   *++yyvsp = yylval;
 
 
-  /* Shift the error token. */
-  YY_SYMBOL_PRINT ("Shifting", yystos[yyn], yyvsp, yylsp);
-
   yystate = yyn;
   goto yynewstate;
 
@@ -2034,25 +1886,16 @@ yyabortlab:
   goto yyreturn;
 
 #ifndef yyoverflow
-/*-------------------------------------------------.
-| yyexhaustedlab -- memory exhaustion comes here.  |
-`-------------------------------------------------*/
-yyexhaustedlab:
-  yyerror (YY_("memory exhausted"));
+/*----------------------------------------------.
+| yyoverflowlab -- parser overflow comes here.  |
+`----------------------------------------------*/
+yyoverflowlab:
+  yyerror ("parser stack overflow");
   yyresult = 2;
   /* Fall through.  */
 #endif
 
 yyreturn:
-  if (yychar != YYEOF && yychar != YYEMPTY)
-     yydestruct ("Cleanup: discarding lookahead",
-		 yytoken, &yylval);
-  while (yyssp != yyss)
-    {
-      yydestruct ("Cleanup: popping",
-		  yystos[*yyssp], yyvsp);
-      YYPOPSTACK;
-    }
 #ifndef yyoverflow
   if (yyss != yyssa)
     YYSTACK_FREE (yyss);
