@@ -31,11 +31,13 @@
 
 #include "elfcpp.h"
 #include "options.h"
+#include "mapfile.h"
 #include "fileread.h"
 #include "readsyms.h"
 #include "symtab.h"
 #include "object.h"
 #include "archive.h"
+#include "plugin.h"
 
 namespace gold
 {
@@ -62,6 +64,11 @@ struct Archive::Archive_header
   char ar_fmag[2];
 };
 
+// Class Archive static variables.
+unsigned int Archive::total_archives;
+unsigned int Archive::total_members;
+unsigned int Archive::total_members_loaded;
+
 // Archive methods.
 
 const char Archive::armag[sarmag] =
@@ -80,7 +87,7 @@ const char Archive::arfmag[2] = { '`', '\n' };
 // table.
 
 void
-Archive::setup()
+Archive::setup(Input_objects* input_objects)
 {
   // We need to ignore empty archives.
   if (this->input_file_->file().filesize() == sarmag)
@@ -116,6 +123,16 @@ Archive::setup()
       const char* px = reinterpret_cast<const char*>(p);
       this->extended_names_.assign(px, extended_size);
     }
+  bool preread_syms = (parameters->options().threads()
+                       && parameters->options().preread_archive_symbols());
+#ifndef ENABLE_THREADS
+  preread_syms = false;
+#else
+  if (parameters->options().has_plugins())
+    preread_syms = false;
+#endif
+  if (preread_syms)
+    this->read_all_symbols(input_objects);
 }
 
 // Unlock any nested archives.
@@ -136,6 +153,12 @@ Archive::unlock_nested_archives()
 void
 Archive::read_armap(off_t start, section_size_type size)
 {
+  // To count the total number of archive members, we'll just count
+  // the number of times the file offset changes.  Since most archives
+  // group the symbols in the armap by object, this ought to give us
+  // an accurate count.
+  off_t last_seen_offset = -1;
+
   // Read in the entire armap.
   const unsigned char* p = this->get_view(start, size, true, false);
 
@@ -159,6 +182,11 @@ Archive::read_armap(off_t start, section_size_type size)
       this->armap_[i].file_offset = elfcpp::Swap<32, true>::readval(pword);
       name_offset += strlen(pnames + name_offset) + 1;
       ++pword;
+      if (this->armap_[i].file_offset != last_seen_offset)
+        {
+          last_seen_offset = this->armap_[i].file_offset;
+          ++this->num_members_;
+        }
     }
 
   if (static_cast<section_size_type>(name_offset) > names_size)
@@ -190,7 +218,7 @@ Archive::read_header(off_t off, bool cache, std::string* pname,
 
 off_t
 Archive::interpret_header(const Archive_header* hdr, off_t off,
-                          std::string* pname, off_t* nested_off)
+                          std::string* pname, off_t* nested_off) const
 {
   if (memcmp(hdr->ar_fmag, arfmag, sizeof arfmag) != 0)
     {
@@ -277,6 +305,301 @@ Archive::interpret_header(const Archive_header* hdr, off_t off,
   return member_size;
 }
 
+// An archive member iterator.
+
+class Archive::const_iterator
+{
+ public:
+  // The header of an archive member.  This is what this iterator
+  // points to.
+  struct Header
+  {
+    // The name of the member.
+    std::string name;
+    // The file offset of the member.
+    off_t off;
+    // The file offset of a nested archive member.
+    off_t nested_off;
+    // The size of the member.
+    off_t size;
+  };
+
+  const_iterator(Archive* archive, off_t off)
+    : archive_(archive), off_(off)
+  { this->read_next_header(); }
+
+  const Header&
+  operator*() const
+  { return this->header_; }
+
+  const Header*
+  operator->() const
+  { return &this->header_; }
+
+  const_iterator&
+  operator++()
+  {
+    if (this->off_ == this->archive_->file().filesize())
+      return *this;
+    this->off_ += sizeof(Archive_header);
+    if (!this->archive_->is_thin_archive())
+      this->off_ += this->header_.size;
+    if ((this->off_ & 1) != 0)
+      ++this->off_;
+    this->read_next_header();
+    return *this;
+  }
+
+  const_iterator
+  operator++(int)
+  {
+    const_iterator ret = *this;
+    ++*this;
+    return ret;
+  }
+
+  bool
+  operator==(const const_iterator p) const
+  { return this->off_ == p->off; }
+
+  bool
+  operator!=(const const_iterator p) const
+  { return this->off_ != p->off; }
+
+ private:
+  void
+  read_next_header();
+
+  // The underlying archive.
+  Archive* archive_;
+  // The current offset in the file.
+  off_t off_;
+  // The current archive header.
+  Header header_;
+};
+
+// Read the next archive header.
+
+void
+Archive::const_iterator::read_next_header()
+{
+  off_t filesize = this->archive_->file().filesize();
+  while (true)
+    {
+      if (filesize - this->off_ < static_cast<off_t>(sizeof(Archive_header)))
+	{
+	  if (filesize != this->off_)
+	    {
+	      gold_error(_("%s: short archive header at %zu"),
+			 this->archive_->filename().c_str(),
+			 static_cast<size_t>(this->off_));
+	      this->off_ = filesize;
+	    }
+	  this->header_.off = filesize;
+	  return;
+	}
+
+      unsigned char buf[sizeof(Archive_header)];
+      this->archive_->file().read(this->off_, sizeof(Archive_header), buf);
+
+      const Archive_header* hdr = reinterpret_cast<const Archive_header*>(buf);
+      this->header_.size =
+	this->archive_->interpret_header(hdr, this->off_, &this->header_.name,
+					 &this->header_.nested_off);
+      this->header_.off = this->off_;
+
+      // Skip special members.
+      if (!this->header_.name.empty() && this->header_.name != "/")
+	return;
+
+      this->off_ += sizeof(Archive_header) + this->header_.size;
+      if ((this->off_ & 1) != 0)
+	++this->off_;
+    }
+}
+
+// Initial iterator.
+
+Archive::const_iterator
+Archive::begin()
+{
+  return Archive::const_iterator(this, sarmag);
+}
+
+// Final iterator.
+
+Archive::const_iterator
+Archive::end()
+{
+  return Archive::const_iterator(this, this->input_file_->file().filesize());
+}
+
+// Get the file and offset for an archive member, which may be an
+// external member of a thin archive.  Set *INPUT_FILE to the
+// file containing the actual member, *MEMOFF to the offset
+// within that file (0 if not a nested archive), and *MEMBER_NAME
+// to the name of the archive member.  Return TRUE on success.
+
+bool
+Archive::get_file_and_offset(off_t off, Input_objects* input_objects,
+                             Input_file** input_file, off_t* memoff,
+                             off_t* memsize, std::string* member_name)
+{
+  off_t nested_off;
+
+  *memsize = this->read_header(off, false, member_name, &nested_off);
+
+  *input_file = this->input_file_;
+  *memoff = off + static_cast<off_t>(sizeof(Archive_header));
+
+  if (!this->is_thin_archive_)
+    return true;
+
+  // Adjust a relative pathname so that it is relative
+  // to the directory containing the archive.
+  if (!IS_ABSOLUTE_PATH(member_name->c_str()))
+    {
+      const char* arch_path = this->filename().c_str();
+      const char* basename = lbasename(arch_path);
+      if (basename > arch_path)
+        member_name->replace(0, 0,
+                             this->filename().substr(0, basename - arch_path));
+    }
+
+  if (nested_off > 0)
+    {
+      // This is a member of a nested archive.  Open the containing
+      // archive if we don't already have it open, then do a recursive
+      // call to include the member from that archive.
+      Archive* arch;
+      Nested_archive_table::const_iterator p =
+        this->nested_archives_.find(*member_name);
+      if (p != this->nested_archives_.end())
+        arch = p->second;
+      else
+        {
+          Input_file_argument* input_file_arg =
+            new Input_file_argument(member_name->c_str(), false, "", false,
+                                    parameters->options());
+          *input_file = new Input_file(input_file_arg);
+          if (!(*input_file)->open(parameters->options(), *this->dirpath_,
+                                   this->task_))
+            return false;
+          arch = new Archive(*member_name, *input_file, false, this->dirpath_,
+                             this->task_);
+          arch->setup(input_objects);
+          std::pair<Nested_archive_table::iterator, bool> ins =
+            this->nested_archives_.insert(std::make_pair(*member_name, arch));
+          gold_assert(ins.second);
+        }
+      return arch->get_file_and_offset(nested_off, input_objects, input_file,
+                                       memoff, memsize, member_name);
+    }
+
+  // This is an external member of a thin archive.  Open the
+  // file as a regular relocatable object file.
+  Input_file_argument* input_file_arg =
+      new Input_file_argument(member_name->c_str(), false, "", false,
+                              this->input_file_->options());
+  *input_file = new Input_file(input_file_arg);
+  if (!(*input_file)->open(parameters->options(), *this->dirpath_,
+                           this->task_))
+    return false;
+
+  *memoff = 0;
+  *memsize = (*input_file)->file().filesize();
+  return true;
+}
+
+// Return an ELF object for the member at offset OFF.  Set *MEMBER_NAME to
+// the name of the member.
+
+Object*
+Archive::get_elf_object_for_member(off_t off, Input_objects* input_objects)
+{
+  std::string member_name;
+  Input_file* input_file;
+  off_t memoff;
+  off_t memsize;
+
+  if (!this->get_file_and_offset(off, input_objects, &input_file, &memoff,
+                                 &memsize, &member_name))
+    return NULL;
+
+  if (parameters->options().has_plugins())
+    {
+      Object* obj = parameters->options().plugins()->claim_file(input_file,
+                                                                memoff,
+                                                                memsize);
+      if (obj != NULL)
+        {
+          // The input file was claimed by a plugin, and its symbols
+          // have been provided by the plugin.
+	  input_file->file().claim_for_plugin();
+          return obj;
+        }
+    }
+
+  off_t filesize = input_file->file().filesize();
+  int read_size = elfcpp::Elf_sizes<64>::ehdr_size;
+  if (filesize - memoff < read_size)
+    read_size = filesize - memoff;
+
+  if (read_size < 4)
+    {
+      gold_error(_("%s: member at %zu is not an ELF object"),
+		 this->name().c_str(), static_cast<size_t>(off));
+      return NULL;
+    }
+
+  const unsigned char* ehdr = input_file->file().get_view(memoff, 0, read_size,
+							  true, false);
+
+  static unsigned char elfmagic[4] =
+    {
+      elfcpp::ELFMAG0, elfcpp::ELFMAG1,
+      elfcpp::ELFMAG2, elfcpp::ELFMAG3
+    };
+  if (memcmp(ehdr, elfmagic, 4) != 0)
+    {
+      gold_error(_("%s: member at %zu is not an ELF object"),
+		 this->name().c_str(), static_cast<size_t>(off));
+      return NULL;
+    }
+
+  return make_elf_object((std::string(this->input_file_->filename())
+				 + "(" + member_name + ")"),
+				input_file, memoff, ehdr, read_size);
+}
+
+// Read the symbols from all the archive members in the link.
+
+void
+Archive::read_all_symbols(Input_objects* input_objects)
+{
+  for (Archive::const_iterator p = this->begin();
+       p != this->end();
+       ++p)
+    this->read_symbols(input_objects, p->off);
+}
+
+// Read the symbols from an archive member in the link.  OFF is the file
+// offset of the member header.
+
+void
+Archive::read_symbols(Input_objects* input_objects, off_t off)
+{
+  Object* obj = this->get_elf_object_for_member(off, input_objects);
+
+  if (obj == NULL)
+    return;
+
+  Read_symbols_data* sd = new Read_symbols_data;
+  obj->read_symbols(sd);
+  Archive_member member(obj, sd);
+  this->members_[off] = member;
+}
+
 // Select members from the archive and add them to the link.  We walk
 // through the elements in the archive map, and look each one up in
 // the symbol table.  If it exists as a strong undefined symbol, we
@@ -286,10 +609,17 @@ Archive::interpret_header(const Archive_header* hdr, off_t off,
 
 void
 Archive::add_symbols(Symbol_table* symtab, Layout* layout,
-		     Input_objects* input_objects)
+		     Input_objects* input_objects, Mapfile* mapfile)
 {
+  ++Archive::total_archives;
+
   if (this->input_file_->options().whole_archive())
-    return this->include_all_members(symtab, layout, input_objects);
+    return this->include_all_members(symtab, layout, input_objects,
+				     mapfile);
+
+  Archive::total_members += this->num_members_;
+
+  input_objects->archive_start(this);
 
   const size_t armap_size = this->armap_.size();
 
@@ -343,180 +673,143 @@ Archive::add_symbols(Symbol_table* symtab, Layout* layout,
 	  last_seen_offset = this->armap_[i].file_offset;
 	  this->seen_offsets_.insert(last_seen_offset);
           this->armap_checked_[i] = true;
+
+	  std::string why;
+	  if (sym == NULL)
+	    {
+	      why = "-u ";
+	      why += sym_name;
+	    }
 	  this->include_member(symtab, layout, input_objects,
-			       last_seen_offset);
+			       last_seen_offset, mapfile, sym, why.c_str());
+
 	  added_new_object = true;
 	}
     }
   while (added_new_object);
+
+  input_objects->archive_stop(this);
 }
 
 // Include all the archive members in the link.  This is for --whole-archive.
 
 void
 Archive::include_all_members(Symbol_table* symtab, Layout* layout,
-                             Input_objects* input_objects)
+                             Input_objects* input_objects, Mapfile* mapfile)
 {
-  off_t off = sarmag;
-  off_t filesize = this->input_file_->file().filesize();
-  while (true)
+  input_objects->archive_start(this);
+
+  if (this->members_.size() > 0)
     {
-      if (filesize - off < static_cast<off_t>(sizeof(Archive_header)))
+      std::map<off_t, Archive_member>::const_iterator p;
+      for (p = this->members_.begin();
+           p != this->members_.end();
+           ++p)
         {
-          if (filesize != off)
-	    gold_error(_("%s: short archive header at %zu"),
-		       this->name().c_str(), static_cast<size_t>(off));
-          break;
+          this->include_member(symtab, layout, input_objects, p->first,
+			       mapfile, NULL, "--whole-archive");
+          ++Archive::total_members;
         }
-
-      unsigned char hdr_buf[sizeof(Archive_header)];
-      this->input_file_->file().read(off, sizeof(Archive_header), hdr_buf);
-
-      const Archive_header* hdr =
-	reinterpret_cast<const Archive_header*>(hdr_buf);
-      std::string name;
-      off_t size = this->interpret_header(hdr, off, &name, NULL);
-      if (name.empty())
-        {
-          // Symbol table.
-        }
-      else if (name == "/")
-        {
-          // Extended name table.
-        }
-      else
-        this->include_member(symtab, layout, input_objects, off);
-
-      off += sizeof(Archive_header);
-      if (!this->is_thin_archive_)
-        off += size;
-      if ((off & 1) != 0)
-        ++off;
-    }
-}
-
-// Include an archive member in the link.  OFF is the file offset of
-// the member header.
-
-void
-Archive::include_member(Symbol_table* symtab, Layout* layout,
-			Input_objects* input_objects, off_t off)
-{
-  std::string n;
-  off_t nested_off;
-  this->read_header(off, false, &n, &nested_off);
-
-  Input_file* input_file;
-  off_t memoff;
-
-  if (!this->is_thin_archive_)
-    {
-      input_file = this->input_file_;
-      memoff = off + static_cast<off_t>(sizeof(Archive_header));
     }
   else
     {
-      // Adjust a relative pathname so that it is relative
-      // to the directory containing the archive.
-      if (!IS_ABSOLUTE_PATH(n.c_str()))
+      for (Archive::const_iterator p = this->begin();
+           p != this->end();
+           ++p)
         {
-          const char *arch_path = this->name().c_str();
-          const char *basename = lbasename(arch_path);
-          if (basename > arch_path)
-            n.replace(0, 0, this->name().substr(0, basename - arch_path));
+          this->include_member(symtab, layout, input_objects, p->off,
+			       mapfile, NULL, "--whole-archive");
+          ++Archive::total_members;
         }
-      if (nested_off > 0)
-        {
-          // This is a member of a nested archive.  Open the containing
-          // archive if we don't already have it open, then do a recursive
-          // call to include the member from that archive.
-          Archive* arch;
-          Nested_archive_table::const_iterator p =
-            this->nested_archives_.find(n);
-          if (p != this->nested_archives_.end())
-            arch = p->second;
-          else
-            {
-              Input_file_argument* input_file_arg =
-                new Input_file_argument(n.c_str(), false, "", false,
-                                        parameters->options());
-              input_file = new Input_file(input_file_arg);
-              if (!input_file->open(parameters->options(), *this->dirpath_,
-                                    this->task_))
-                return;
-              arch = new Archive(n, input_file, false, this->dirpath_,
-                                 this->task_);
-              arch->setup();
-              std::pair<Nested_archive_table::iterator, bool> ins =
-                this->nested_archives_.insert(std::make_pair(n, arch));
-              gold_assert(ins.second);
-            }
-          arch->include_member(symtab, layout, input_objects, nested_off);
-          return;
-        }
-      // This is an external member of a thin archive.  Open the
-      // file as a regular relocatable object file.
-      Input_file_argument* input_file_arg =
-          new Input_file_argument(n.c_str(), false, "", false,
-                                  this->input_file_->options());
-      input_file = new Input_file(input_file_arg);
-      if (!input_file->open(parameters->options(), *this->dirpath_,
-                            this->task_))
-        {
-          return;
-        }
-      memoff = 0;
     }
 
-  off_t filesize = input_file->file().filesize();
-  int read_size = elfcpp::Elf_sizes<64>::ehdr_size;
-  if (filesize - memoff < read_size)
-    read_size = filesize - memoff;
+  input_objects->archive_stop(this);
+}
 
-  if (read_size < 4)
+// Return the number of members in the archive.  This is only used for
+// reports.
+
+size_t
+Archive::count_members()
+{
+  size_t ret = 0;
+  for (Archive::const_iterator p = this->begin();
+       p != this->end();
+       ++p)
+    ++ret;
+  return ret;
+}
+
+// Include an archive member in the link.  OFF is the file offset of
+// the member header.  WHY is the reason we are including this member.
+
+void
+Archive::include_member(Symbol_table* symtab, Layout* layout,
+			Input_objects* input_objects, off_t off,
+			Mapfile* mapfile, Symbol* sym, const char* why)
+{
+  ++Archive::total_members_loaded;
+
+  std::map<off_t, Archive_member>::const_iterator p = this->members_.find(off);
+  if (p != this->members_.end())
     {
-      gold_error(_("%s: member at %zu is not an ELF object"),
-		 this->name().c_str(), static_cast<size_t>(off));
+      Object *obj = p->second.obj_;
+      Read_symbols_data *sd = p->second.sd_;
+      if (mapfile != NULL)
+        mapfile->report_include_archive_member(obj->name(), sym, why);
+      if (input_objects->add_object(obj))
+        {
+          obj->layout(symtab, layout, sd);
+          obj->add_symbols(symtab, sd);
+        }
+      delete sd;
       return;
     }
 
-  const unsigned char* ehdr = input_file->file().get_view(memoff, 0, read_size,
-							  true, false);
+  Object* obj = this->get_elf_object_for_member(off, input_objects);
+  if (obj == NULL)
+    return;
 
-  static unsigned char elfmagic[4] =
+  if (mapfile != NULL)
+    mapfile->report_include_archive_member(obj->name(), sym, why);
+
+  Pluginobj* pluginobj = obj->pluginobj();
+  if (pluginobj != NULL)
     {
-      elfcpp::ELFMAG0, elfcpp::ELFMAG1,
-      elfcpp::ELFMAG2, elfcpp::ELFMAG3
-    };
-  if (memcmp(ehdr, elfmagic, 4) != 0)
-    {
-      gold_error(_("%s: member at %zu is not an ELF object"),
-		 this->name().c_str(), static_cast<size_t>(off));
+      pluginobj->add_symbols(symtab, layout);
       return;
     }
 
-  Object* obj = make_elf_object((std::string(this->input_file_->filename())
-				 + "(" + n + ")"),
-				input_file, memoff, ehdr, read_size);
-
   if (input_objects->add_object(obj))
     {
       Read_symbols_data sd;
       obj->read_symbols(&sd);
       obj->layout(symtab, layout, &sd);
       obj->add_symbols(symtab, &sd);
+
+      // If this is an external member of a thin archive, unlock the file
+      // for the next task.
+      if (obj->offset() == 0)
+        obj->unlock(this->task_);
     }
   else
     {
       // FIXME: We need to close the descriptor here.
       delete obj;
     }
+}
 
-  if (this->is_thin_archive_)
-    {
-      // Opening the file locked it.  Unlock it now.
-      input_file->file().unlock(this->task_);
-    }
+// Print statistical information to stderr.  This is used for --stats.
+
+void
+Archive::print_stats()
+{
+  fprintf(stderr, _("%s: archive libraries: %u\n"),
+          program_name, Archive::total_archives);
+  fprintf(stderr, _("%s: total archive members: %u\n"),
+          program_name, Archive::total_members);
+  fprintf(stderr, _("%s: loaded archive members: %u\n"),
+          program_name, Archive::total_members_loaded);
 }
 
 // Add_archive_symbols methods.
@@ -551,7 +844,7 @@ void
 Add_archive_symbols::run(Workqueue*)
 {
   this->archive_->add_symbols(this->symtab_, this->layout_,
-			      this->input_objects_);
+			      this->input_objects_, this->mapfile_);
 
   this->archive_->unlock_nested_archives();
 
