@@ -36,6 +36,7 @@
 #include "script.h"
 #include "target-select.h"
 #include "options.h"
+#include "plugin.h"
 
 namespace gold
 {
@@ -296,6 +297,15 @@ General_options::parse_library(const char*, const char* arg,
   cmdline->inputs().add_file(file);
 }
 
+#ifdef ENABLE_PLUGINS
+void
+General_options::parse_plugin(const char*, const char* arg,
+                              Command_line*)
+{
+  this->add_plugin(arg);
+}
+#endif // ENABLE_PLUGINS
+
 void
 General_options::parse_R(const char* option, const char* arg,
                          Command_line* cmdline)
@@ -337,6 +347,14 @@ General_options::parse_version_script(const char*, const char* arg,
     gold::gold_fatal(_("unable to parse version script file %s"), arg);
 }
 
+void
+General_options::parse_dynamic_list(const char*, const char* arg,
+                                    Command_line* cmdline)
+{
+  if (!read_dynamic_list(arg, cmdline, &this->dynamic_list_))
+    gold::gold_fatal(_("unable to parse dynamic-list script file %s"), arg);
+}
+
 void
 General_options::parse_start_group(const char*, const char*,
                                    Command_line* cmdline)
@@ -594,7 +612,7 @@ namespace gold
 
 General_options::General_options()
   : execstack_status_(General_options::EXECSTACK_FROM_INPUT), static_(false),
-    do_demangle_(false)
+    do_demangle_(false), plugins_()
 {
 }
 
@@ -632,6 +650,16 @@ General_options::add_sysroot()
   free(canonical_sysroot);
 }
 
+// Add a plugin and its arguments to the list of plugins.
+
+void
+General_options::add_plugin(const char* arg)
+{
+  if (this->plugins_ == NULL)
+    this->plugins_ = new Plugin_manager(*this);
+  this->plugins_->add_plugin(arg);
+}
+
 // Set up variables and other state that isn't set up automatically by
 // the parse routine, and ensure options don't contradict each other
 // and are otherwise kosher.
@@ -772,6 +800,9 @@ General_options::finalize()
   this->add_sysroot();
 
   // Now that we've normalized the options, check for contradictory ones.
+  if (this->shared() && this->is_static())
+    gold_fatal(_("-shared and -static are incompatible"));
+
   if (this->shared() && this->relocatable())
     gold_fatal(_("-shared and -r are incompatible"));
 
