@@ -32,6 +32,7 @@
 #include "archive.h"
 #include "script.h"
 #include "readsyms.h"
+#include "plugin.h"
 
 namespace gold
 {
@@ -159,6 +160,53 @@ Read_symbols::do_read_symbols(Workqueue* workqueue)
   const unsigned char* ehdr = input_file->file().get_view(0, 0, read_size,
 							  true, false);
 
+  if (read_size >= Archive::sarmag)
+    {
+      bool is_thin_archive
+          = memcmp(ehdr, Archive::armagt, Archive::sarmag) == 0;
+      if (is_thin_archive 
+          || memcmp(ehdr, Archive::armag, Archive::sarmag) == 0)
+	{
+	  // This is an archive.
+	  Archive* arch = new Archive(this->input_argument_->file().name(),
+				      input_file, is_thin_archive,
+				      this->dirpath_, this);
+	  arch->setup(this->input_objects_);
+	  
+	  // Unlock the archive so it can be used in the next task.
+	  arch->unlock(this);
+
+	  workqueue->queue_next(new Add_archive_symbols(this->symtab_,
+							this->layout_,
+							this->input_objects_,
+							this->mapfile_,
+							arch,
+							this->input_group_,
+							this->this_blocker_,
+							this->next_blocker_));
+	  return true;
+	}
+    }
+
+  if (parameters->options().has_plugins())
+    {
+      Pluginobj* obj = parameters->options().plugins()->claim_file(input_file,
+                                                                   0, filesize);
+      if (obj != NULL)
+        {
+          // The input file was claimed by a plugin, and its symbols
+          // have been provided by the plugin.
+	  input_file->file().claim_for_plugin();
+	  input_file->file().unlock(this);
+          workqueue->queue_next(new Add_plugin_symbols(this->symtab_,
+                                                       this->layout_,
+                                                       obj,
+                                                       this->this_blocker_,
+                                                       this->next_blocker_));
+          return true;
+        }
+    }
+
   if (read_size >= 4)
     {
       static unsigned char elfmagic[4] =
@@ -201,33 +249,6 @@ Read_symbols::do_read_symbols(Workqueue* workqueue)
 	}
     }
 
-  if (read_size >= Archive::sarmag)
-    {
-      bool is_thin_archive
-          = memcmp(ehdr, Archive::armagt, Archive::sarmag) == 0;
-      if (is_thin_archive 
-          || memcmp(ehdr, Archive::armag, Archive::sarmag) == 0)
-	{
-	  // This is an archive.
-	  Archive* arch = new Archive(this->input_argument_->file().name(),
-				      input_file, is_thin_archive,
-				      this->dirpath_, this);
-	  arch->setup();
-	  
-	  // Unlock the archive so it can be used in the next task.
-	  arch->unlock(this);
-
-	  workqueue->queue_next(new Add_archive_symbols(this->symtab_,
-							this->layout_,
-							this->input_objects_,
-							arch,
-							this->input_group_,
-							this->this_blocker_,
-							this->next_blocker_));
-	  return true;
-	}
-    }
-
   // Queue up a task to try to parse this file as a script.  We use a
   // separate task so that the script will be read in order with other
   // objects named on the command line.  Also so that we don't try to
@@ -239,6 +260,7 @@ Read_symbols::do_read_symbols(Workqueue* workqueue)
 					this->layout_,
 					this->dirpath_,
 					this->input_objects_,
+					this->mapfile_,
 					this->input_group_,
 					this->input_argument_,
 					input_file,
@@ -274,7 +296,8 @@ Read_symbols::do_group(Workqueue* workqueue)
       workqueue->queue_soon(new Read_symbols(this->options_,
 					     this->input_objects_,
 					     this->symtab_, this->layout_,
-					     this->dirpath_, arg, input_group,
+					     this->dirpath_, this->mapfile_,
+					     arg, input_group,
 					     this_blocker, next_blocker));
       this_blocker = next_blocker;
     }
@@ -283,6 +306,7 @@ Read_symbols::do_group(Workqueue* workqueue)
   workqueue->queue_soon(new Finish_group(this->input_objects_,
 					 this->symtab_,
 					 this->layout_,
+					 this->mapfile_,
 					 input_group,
 					 saw_undefined,
 					 this_blocker,
@@ -411,7 +435,7 @@ Finish_group::run(Workqueue*)
 	  Task_lock_obj<Archive> tl(this, *p);
 
 	  (*p)->add_symbols(this->symtab_, this->layout_,
-			    this->input_objects_);
+			    this->input_objects_, this->mapfile_);
 	}
     }
 
@@ -459,9 +483,9 @@ Read_script::run(Workqueue* workqueue)
   bool used_next_blocker;
   if (!read_input_script(workqueue, this->options_, this->symtab_,
 			 this->layout_, this->dirpath_, this->input_objects_,
-			 this->input_group_, this->input_argument_,
-			 this->input_file_, this->next_blocker_,
-			 &used_next_blocker))
+			 this->mapfile_, this->input_group_,
+			 this->input_argument_, this->input_file_,
+			 this->next_blocker_, &used_next_blocker))
     {
       // Here we have to handle any other input file types we need.
       gold_error(_("%s: not an object or archive"),
