@@ -43,6 +43,7 @@ class General_options;
 class Command_line;
 class Symbol_table;
 class Layout;
+class Mapfile;
 class Input_argument;
 class Input_objects;
 class Input_group;
@@ -137,22 +138,17 @@ class Version_script_info
   empty() const
   { return this->version_trees_.empty(); }
 
-  // Return the version associated with the given symbol name.
-  // Strings are allocated out of the stringpool given in the
-  // constructor.  Strings are allocated out of the stringpool given
-  // in the constructor.
-  const std::string&
-  get_symbol_version(const char* symbol) const
-  { return get_symbol_version_helper(symbol, true); }
+  // If there is a version associated with SYMBOL, return true, and
+  // set *VERSION to the version.  Otherwise, return false.
+  bool
+  get_symbol_version(const char* symbol, std::string* version) const
+  { return this->get_symbol_version_helper(symbol, true, version); }
 
-  // Return whether this symbol matches the local: section of a
-  // version script (it doesn't matter which).
+  // Return whether this symbol matches the local: section of some
+  // version.
   bool
   symbol_is_local(const char* symbol) const
-  {
-    return (get_symbol_version(symbol).empty()
-            && !get_symbol_version_helper(symbol, false).empty());
-  }
+  { return this->get_symbol_version_helper(symbol, false, NULL); }
 
   // Return the names of versions defined in the version script.
   // Strings are allocated out of the stringpool given in the
@@ -185,8 +181,9 @@ class Version_script_info
   void
   print_expression_list(FILE* f, const Version_expression_list*) const;
 
-  const std::string& get_symbol_version_helper(const char* symbol,
-                                               bool check_global) const;
+  bool get_symbol_version_helper(const char* symbol,
+				 bool check_global,
+				 std::string* pversion) const;
 
   std::vector<struct Version_dependency_list*> dependency_lists_;
   std::vector<struct Version_expression_list*> expression_lists_;
@@ -393,7 +390,7 @@ class Script_options
 
 bool
 read_input_script(Workqueue*, const General_options&, Symbol_table*, Layout*,
-		  Dirsearch*, Input_objects*, Input_group*,
+		  Dirsearch*, Input_objects*, Mapfile*, Input_group*,
 		  const Input_argument*, Input_file*,
 		  Task_token* next_blocker, bool* used_next_blocker);
 
@@ -401,7 +398,7 @@ read_input_script(Workqueue*, const General_options&, Symbol_table*, Layout*,
 // Read it as a script, and execute its contents immediately.
 
 bool
-read_commandline_script(const char* filename, Command_line*);
+read_commandline_script(const char* filename, Command_line* cmdline);
 
 // FILE was found as an argument to --version-script.  Read it as a
 // version script, and store its contents in
@@ -410,6 +407,13 @@ read_commandline_script(const char* filename, Command_line*);
 bool
 read_version_script(const char* filename, Command_line* cmdline);
 
+// FILENAME was found as an argument to --dynamic-list.  Read it as a
+// version script (actually, a versym_node from a version script), and
+// store its contents in DYNAMIC_LIST.
+
+bool
+read_dynamic_list(const char* filename, Command_line* cmdline,
+                  Script_options* dynamic_list);
 
 } // End namespace gold.
 
