@@ -2,6 +2,7 @@ SCRIPT_NAME=elf
 TEMPLATE_NAME=elf32
 EXTRA_EM_FILE=spuelf
 OUTPUT_FORMAT="elf32-spu"
+NO_REL_RELOCS=yes
 ARCH=spu
 MACHINE=
 ALIGNMENT=16
@@ -18,4 +19,4 @@ MAXPAGESIZE=0x80
 DATA_ADDR="ALIGN(${MAXPAGESIZE})"
 OTHER_BSS_SECTIONS=".toe ALIGN(128) : { *(.toe) } = 0"
 OTHER_SECTIONS=".note.spu_name 0 : { KEEP(*(.note.spu_name)) }
-  ._ea 0 : { KEEP(*(._ea)) }"
+  ._ea 0 : { KEEP(*(._ea)) KEEP(*(._ea.*)) }"
