@@ -1,6 +1,6 @@
 /* Routines to help build PEI-format DLLs (Win32 etc)
    Copyright 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-   2008, 2009 Free Software Foundation, Inc.
+   2008 Free Software Foundation, Inc.
    Written by DJ Delorie <dj@cygnus.com>
 
    This file is part of the GNU Binutils.
@@ -40,7 +40,6 @@
 #include "coff/internal.h"
 #include "../bfd/libcoff.h"
 #include "deffile.h"
-#include "pe-dll.h"
 
 #ifdef pe_use_x86_64
 
@@ -165,6 +164,7 @@ static struct bfd_section *edata_s, *reloc_s;
 static unsigned char *edata_d, *reloc_d;
 static size_t edata_sz, reloc_sz;
 static int runtime_pseudo_relocs_created = 0;
+static int runtime_pseudp_reloc_v2_init = 0;
 
 typedef struct
 {
@@ -314,7 +314,6 @@ static const autofilter_entry_type autofilter_liblist[] =
   { STRING_COMMA_LEN ("libcegcc") },
   { STRING_COMMA_LEN ("libcygwin") },
   { STRING_COMMA_LEN ("libgcc") },
-  { STRING_COMMA_LEN ("libgcc_s") },
   { STRING_COMMA_LEN ("libstdc++") },
   { STRING_COMMA_LEN ("libmingw32") },
   { STRING_COMMA_LEN ("libmingwex") },
@@ -325,37 +324,6 @@ static const autofilter_entry_type autofilter_liblist[] =
   { NULL, 0 }
 };
 
-/* Regardless of the suffix issue mentioned above, we must ensure that
-  we do not falsely match on a leading substring, such as when libtool
-  builds libstdc++ as a DLL using libsupc++convenience.a as an intermediate.
-  This routine ensures that the leading part of the name matches and that
-  it is followed by only an optional version suffix and a file extension,
-  returning zero if so or -1 if not.  */
-static int libnamencmp (const char *libname, const autofilter_entry_type *afptr)
-{
-  if (strncmp (libname, afptr->name, afptr->len))
-    return -1;
-
-  libname += afptr->len;
-
-  /* Be liberal in interpreting what counts as a version suffix; we
-    accept anything that has a dash to separate it from the name and
-    begins with a digit.  */
-  if (libname[0] == '-')
-    {
-      if (!ISDIGIT (*++libname))
-	return -1;
-      /* Ensure the filename has an extension.  */
-      while (*++libname != '.')
-	if (!*libname)
-	  return -1;
-    }
-  else if (libname[0] != '.')
-    return -1;
-
-  return 0;
-}
-
 static const autofilter_entry_type autofilter_objlist[] =
 {
   { STRING_COMMA_LEN ("crt0.o") },
@@ -533,7 +501,7 @@ auto_export (bfd *abfd, def_file *d, const char *n)
 
 	  while (afptr->name)
 	    {
-	      if (libnamencmp (libname, afptr) == 0 )
+	      if (strncmp (libname, afptr->name, afptr->len) == 0 )
 		return 0;
 	      afptr++;
 	    }
@@ -2274,6 +2242,7 @@ make_import_fixup_entry (const char *name,
   d2 = xmalloc (20);
   id2->contents = d2;
   memset (d2, 0, 20);
+  d2[0] = d2[16] = PE_IDATA5_SIZE; /* Reloc addend.  */
 
   quick_reloc (abfd, 0, BFD_RELOC_RVA, 1);
   quick_reloc (abfd, 12, BFD_RELOC_RVA, 2);
@@ -2295,14 +2264,14 @@ make_import_fixup_entry (const char *name,
 static bfd *
 make_runtime_pseudo_reloc (const char *name ATTRIBUTE_UNUSED,
 			   const char *fixup_name,
-			   int addend,
+			   bfd_vma addend ATTRIBUTE_UNUSED,
+			   bfd_vma bitsize,
 			   bfd *parent)
 {
   asection *rt_rel;
   unsigned char *rt_rel_d;
   char *oname;
   bfd *abfd;
-
   oname = xmalloc (20);
   sprintf (oname, "rtr%06d.o", tmp_seq);
   tmp_seq++;
@@ -2321,19 +2290,47 @@ make_runtime_pseudo_reloc (const char *name ATTRIBUTE_UNUSED,
 
   quick_symbol (abfd, "", fixup_name, "", UNDSEC, BSF_GLOBAL, 0);
 
-  bfd_set_section_size (abfd, rt_rel, 8);
-  rt_rel_d = xmalloc (8);
-  rt_rel->contents = rt_rel_d;
-  memset (rt_rel_d, 0, 8);
-  bfd_put_32 (abfd, addend, rt_rel_d);
+  if (link_info.pei386_runtime_pseudo_reloc == 2)
+    {
+	  size_t size = 12;
+	  if (! runtime_pseudp_reloc_v2_init)
+	    {
+		  size += 12;
+		  runtime_pseudp_reloc_v2_init = 1;
+	    }
+      quick_symbol (abfd, U ("_imp_"), name, "", UNDSEC, BSF_GLOBAL, 0);
 
-  quick_reloc (abfd, 4, BFD_RELOC_RVA, 1);
-  save_relocs (rt_rel);
+      bfd_set_section_size (abfd, rt_rel, size);
+      rt_rel_d = xmalloc (size);
+      rt_rel->contents = rt_rel_d;
+      memset (rt_rel_d, 0, size);
+	  quick_reloc (abfd, size - 8, BFD_RELOC_RVA, 1);
+	  quick_reloc (abfd, size - 12, BFD_RELOC_RVA, 2);
+	  bfd_put_32 (abfd, bitsize, rt_rel_d + (size - 4));
+	  if (size != 12)
+	    bfd_put_32 (abfd, 1, rt_rel_d + 8);
+      save_relocs (rt_rel);
 
-  bfd_set_symtab (abfd, symtab, symptr);
+      bfd_set_symtab (abfd, symtab, symptr);
 
-  bfd_set_section_contents (abfd, rt_rel, rt_rel_d, 0, 8);
+      bfd_set_section_contents (abfd, rt_rel, rt_rel_d, 0, size);
+   }
+  else
+   {
+      bfd_set_section_size (abfd, rt_rel, 8);
+      rt_rel_d = xmalloc (8);
+      rt_rel->contents = rt_rel_d;
+      memset (rt_rel_d, 0, 8);
 
+      bfd_put_32 (abfd, addend, rt_rel_d);
+      quick_reloc (abfd, 4, BFD_RELOC_RVA, 1);
+
+      save_relocs (rt_rel);
+
+      bfd_set_symtab (abfd, symtab, symptr);
+
+      bfd_set_section_contents (abfd, rt_rel, rt_rel_d, 0, 8);
+   }
   bfd_make_readable (abfd);
   return abfd;
 }
@@ -2383,7 +2380,7 @@ pe_create_runtime_relocator_reference (bfd *parent)
 }
 
 void
-pe_create_import_fixup (arelent *rel, asection *s, int addend)
+pe_create_import_fixup (arelent *rel, asection *s, bfd_vma addend)
 {
   char buf[300];
   struct bfd_symbol *sym = *rel->sym_ptr_ptr;
@@ -2416,31 +2413,30 @@ pe_create_import_fixup (arelent *rel, asection *s, int addend)
       add_bfd_to_link (b, b->filename, &link_info);
     }
 
-  if (addend != 0)
-    {
-      if (link_info.pei386_runtime_pseudo_reloc)
-	{
-	  if (pe_dll_extra_pe_debug)
-	    printf ("creating runtime pseudo-reloc entry for %s (addend=%d)\n",
-		   fixup_name, addend);
-	  b = make_runtime_pseudo_reloc (name, fixup_name, addend,
-					 link_info.output_bfd);
-	  add_bfd_to_link (b, b->filename, &link_info);
+    if ((link_info.pei386_runtime_pseudo_reloc != 0 && addend != 0)
+        || link_info.pei386_runtime_pseudo_reloc == 2)
+      {
+	if (pe_dll_extra_pe_debug)
+	  printf ("creating runtime pseudo-reloc entry for %s (addend=%d)\n",
+	          fixup_name, (int) addend);
 
-	  if (runtime_pseudo_relocs_created == 0)
-	    {
-	      b = pe_create_runtime_relocator_reference (link_info.output_bfd);
-	      add_bfd_to_link (b, b->filename, &link_info);
-	    }
-	  runtime_pseudo_relocs_created++;
-	}
-      else
-	{
-	  einfo (_("%C: variable '%T' can't be auto-imported. Please read the documentation for ld's --enable-auto-import for details.\n"),
-		 s->owner, s, rel->address, sym->name);
-	  einfo ("%X");
-	}
-    }
+	b = make_runtime_pseudo_reloc (name, fixup_name, addend, rel->howto->bitsize,
+				       link_info.output_bfd);
+	add_bfd_to_link (b, b->filename, &link_info);
+
+	if (runtime_pseudo_relocs_created == 0)
+	  {
+	    b = pe_create_runtime_relocator_reference (link_info.output_bfd);
+	    add_bfd_to_link (b, b->filename, &link_info);
+	  }
+	runtime_pseudo_relocs_created++;
+      }
+    else if (addend != 0)
+      {
+	einfo (_("%C: variable '%T' can't be auto-imported. Please read the documentation for ld's --enable-auto-import for details.\n"),
+	       s->owner, s, rel->address, sym->name);
+	einfo ("%X");
+      }
 }
 
 
